import { Hono } from "hono";
import { logger } from "hono/logger";
import { Store } from "@willsoto/node-konfig-core";
import { bearerAuth } from "hono/bearer-auth";
import { Counter, Gauge, Registry } from "prom-client";

import youtubeApiPlayer from "./youtube_api_routes/player.ts";
import invidiousRouteLatestVersion from "./invidious_routes/latestVersion.ts";
import invidiousRouteDashManifest from "./invidious_routes/dashManifest.ts";
import videoPlaybackProxy from "./videoPlaybackProxy.ts";
import metrics from "./metrics.ts";

const METRICS_PREFIX = "invidious_companion_";
export const register = new Registry();

export const externalTokenGeneratorFail = new Counter({
    name: `${METRICS_PREFIX}externaltokengenerator_fail`,
    help: "TODO",
    registers: [register],
});

export const cachedEntries = new Gauge({
    name: `${METRICS_PREFIX}cached_entries`,
    help: "TODO",
    registers: [register],
});

export const subreasonProtectCommunity = new Counter({
    name: `${METRICS_PREFIX}subreason_protect_community`,
    help: "TODO",
    registers: [register],
});

export const poTokenFail = new Counter({
    name: `${METRICS_PREFIX}potoken_fail`,
    help: "TODO",
    registers: [register],
});

export const reasonBot = new Counter({
    name: `${METRICS_PREFIX}reason_bot`,
    help: "TODO",
    registers: [register],
});

export const videoUnavailable = new Counter({
    name: `${METRICS_PREFIX}video_unavailable`,
    help: "TODO",
    registers: [register],
});

export const videoRestricted = new Counter({
    name: `${METRICS_PREFIX}video_restricted`,
    help: "TODO",
    registers: [register],
});

export const failedRequests = new Counter({
    name: `${METRICS_PREFIX}failed_requests`,
    help: "TODO",
    registers: [register],
});

export const successfulRequests = new Counter({
    name: `${METRICS_PREFIX}successful_requests`,
    help: "TODO",
    registers: [register],
});

import health from "./health.ts";

export const routes = (
    app: Hono,
    konfigStore: Store<Record<string, unknown>>,
) => {
    app.use("*", logger());

    app.use(
        "/youtubei/v1/*",
        bearerAuth({
            token: Deno.env.get("SERVER_SECRET_KEY") ||
                konfigStore.get("server.secret_key") as string,
        }),
    );

    app.route("/youtubei/v1", youtubeApiPlayer);
    app.route("/latest_version", invidiousRouteLatestVersion);
    app.route("/api/manifest/dash/id", invidiousRouteDashManifest);
    app.route("/videoplayback", videoPlaybackProxy);
    app.route("/metrics", metrics);
    app.route("/healthz", health);
};
